﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Problems.MetaOptimization {
  [StorableClass]
  public class ConstrainedItemList<T> : ItemList<T> where T : class, IItem {
    [Storable]
    private Type type;
    public Type Type {
      get { return type; }
      set {
        if (type != value) {
          type = value;
          this.RemoveAll(x => x.GetType() != type);
        }
      }
    }

    #region Constructors and Cloning
    [StorableConstructor]
    protected ConstrainedItemList(bool deserializing) : base(deserializing) { }
    protected ConstrainedItemList(ConstrainedItemList<T> original, Cloner cloner)
      : base(original, cloner) {
      this.Type = original.Type;
    }
    public ConstrainedItemList()
      : base() {
      Type = typeof(T);
    }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new ConstrainedItemList<T>(this, cloner);
    }
    #endregion
  }
}
